@@ -15,9 +15,7 @@
 # limitations under the License.
 from __future__ import annotations
 
-import contextvars
 import datetime
-import json
 import pathlib
 import tempfile
 from types import TracebackType
@@ -26,62 +24,22 @@ from typing import Any, Dict, Literal, Optional, Tuple, Type, Union
 import fsspec
 import pydantic
 import sqlalchemy
-import sqlalchemy.orm
 
-ENGINE: contextvars.ContextVar[sqlalchemy.engine.Engine] = contextvars.ContextVar(
-    "cacholote_engine"
-)
-SETTINGS: contextvars.ContextVar[Settings] = contextvars.ContextVar(
-    "cacholote_settings"
-)
+from . import database
+
+_SETTINGS: Optional[Settings] = None
 _DEFAULT_CACHE_DIR = pathlib.Path(tempfile.gettempdir()) / "cacholote"
 _DEFAULT_CACHE_DIR.mkdir(exist_ok=True)
 
-Base = sqlalchemy.orm.declarative_base()
-
-
-class CacheEntry(Base):
-    __tablename__ = "cache_entries"
-
-    key = sqlalchemy.Column(sqlalchemy.String(56), primary_key=True)
-    expiration = sqlalchemy.Column(
-        sqlalchemy.DateTime, default=datetime.datetime.max, primary_key=True
-    )
-    result = sqlalchemy.Column(sqlalchemy.JSON)
-    timestamp = sqlalchemy.Column(
-        sqlalchemy.DateTime,
-        default=datetime.datetime.utcnow,
-        onupdate=datetime.datetime.utcnow,
-    )
-    counter = sqlalchemy.Column(sqlalchemy.Integer, default=0)
-    tag = sqlalchemy.Column(sqlalchemy.String)
-
-    constraint = sqlalchemy.UniqueConstraint(key, expiration)
-
-    @property
-    def _result_as_string(self) -> str:
-        return json.dumps(self.result)
-
-    @property
-    def _primary_keys(self) -> Dict[str, Any]:
-        return {name: getattr(self, name) for name in ["key", "expiration"]}
-
-    def __repr__(self) -> str:
-        return str(self._primary_keys)
-
-
-@sqlalchemy.event.listens_for(CacheEntry, "before_insert")  # type: ignore[misc]
-def set_epiration_to_max(
-    mapper: sqlalchemy.orm.Mapper,
-    connection: sqlalchemy.engine.Connection,
-    target: CacheEntry,
-) -> None:
-    target.expiration = target.expiration or datetime.datetime.max
+_CONFIG_NOT_SET_MSG = (
+    "Configuration settings have not been set. Run `cacholote.config.reset()`."
+)
 
 
 class Settings(pydantic.BaseSettings):
     use_cache: bool = True
     cache_db_urlpath: str = f"sqlite:///{_DEFAULT_CACHE_DIR / 'cacholote.db'}"
+    create_engine_kwargs: Dict[str, Any] = {}
     cache_files_urlpath: str = f"{_DEFAULT_CACHE_DIR / 'cache_files'}"
     cache_files_urlpath_readonly: Optional[str] = None
     cache_files_storage_options: Dict[str, Any] = {}
@@ -90,22 +48,19 @@ class Settings(pydantic.BaseSettings):
     ] = "application/netcdf"
     io_delete_original: bool = False
     raise_all_encoding_errors: bool = False
-    expiration: Optional[str] = None
+    expiration: Optional[datetime.datetime] = None
     tag: Optional[str] = None
+    return_cache_entry: bool = False
 
-    @pydantic.validator("expiration")
-    def expiration_must_be_isoformat(
-        cls: pydantic.BaseSettings, expiration: Optional[str]
-    ) -> Optional[str]:
-        """Validate expiration."""
-        if isinstance(expiration, str):
-            try:
-                datetime.datetime.fromisoformat(expiration)
-            except ValueError as ex:
-                raise ValueError(
-                    f"{expiration=} is NOT a valid ISO 8601 format"
-                ) from ex
-        return expiration
+    @pydantic.validator("return_cache_entry")
+    def validate_return_cache_entry(
+        cls: pydantic.BaseSettings, return_cache_entry: bool, values: Dict[str, Any]
+    ) -> bool:
+        if return_cache_entry is True and values["use_cache"] is False:
+            raise ValueError(
+                "`use_cache` must be True when `return_cache_entry` is True"
+            )
+        return return_cache_entry
 
     def make_cache_dir(self) -> None:
         fs, _, (urlpath, *_) = fsspec.get_fs_token_paths(
@@ -114,17 +69,28 @@ class Settings(pydantic.BaseSettings):
         )
         fs.mkdirs(urlpath, exist_ok=True)
 
-    def set_engine(self) -> Optional[contextvars.Token]:  # type: ignore[type-arg] # py38 not subscriptable
-        try:
-            engine = ENGINE.get()
-        except LookupError:
-            pass
-        else:
-            if str(engine.url) == self.cache_db_urlpath:
-                return None
-        engine = sqlalchemy.create_engine(self.cache_db_urlpath, future=True)
-        Base.metadata.create_all(engine)
-        return ENGINE.set(engine)
+    def set_engine_and_session(self, force_reset: bool = False) -> None:
+        if (
+            force_reset
+            or database.ENGINE is None
+            or database.SESSIONMAKER is None
+            or str(database.ENGINE.url) != self.cache_db_urlpath
+        ):
+            database._set_engine_and_session(
+                self.cache_db_urlpath, self.create_engine_kwargs
+            )
+
+    @property
+    def engine(self) -> sqlalchemy.engine.Engine:
+        if database.ENGINE is None:
+            raise ValueError(_CONFIG_NOT_SET_MSG)
+        return database.ENGINE
+
+    @property
+    def sessionmaker(self) -> sqlalchemy.orm.sessionmaker:  # type: ignore[type-arg]
+        if database.SESSIONMAKER is None:
+            raise ValueError(_CONFIG_NOT_SET_MSG)
+        return database.SESSIONMAKER
 
     class Config:
         case_sensitive = False
@@ -142,6 +108,8 @@ class set:
         Enable/disable cache.
     cache_db_urlpath: str, default:"sqlite:////system_tmp_dir/cacholote/cacholote.db"
         URL for cache database (driver://user:pass@host/database).
+    create_engine_kwargs: dict, default: {}
+        Keyword arguments for ``sqlalchemy.create_engine``
     cache_files_urlpath: str, default:"/system_tmp_dir/cacholote/cache_files"
         URL for cache files (protocol://location).
     cache_files_storage_options: dict, default: {}
@@ -161,17 +129,24 @@ class set:
     tag: str, optional, default: None
         Tag for the cache entry. If None, do NOT tag.
         Note that existing tags are overwritten.
+    return_cache_entry: bool, default: False
+        Whether to return the cache database entry rather than decoded results.
     """
 
     def __init__(self, **kwargs: Any):
-        old_settings = SETTINGS.get()
-        new_settings = Settings(**{**old_settings.dict(), **kwargs})
-        new_settings.make_cache_dir()
-        self._settings_token = SETTINGS.set(new_settings)
-        self._engine_token = new_settings.set_engine()
+        self._old_engine = database.ENGINE
+        self._old_session = database.SESSIONMAKER
+        self._old_settings = get()
 
-    def __enter__(self) -> None:
-        pass
+        global _SETTINGS
+        _SETTINGS = Settings(**{**self._old_settings.dict(), **kwargs})
+        _SETTINGS.make_cache_dir()
+        _SETTINGS.set_engine_and_session(
+            self._old_settings.create_engine_kwargs != _SETTINGS.create_engine_kwargs
+        )
+
+    def __enter__(self) -> Settings:
+        return get()
 
     def __exit__(
         self,
@@ -179,16 +154,18 @@ class set:
         exc_val: Optional[BaseException],
         exc_tb: Optional[TracebackType],
     ) -> None:
-        if self._engine_token:
-            ENGINE.reset(self._engine_token)
-        SETTINGS.reset(self._settings_token)
+        database.ENGINE = self._old_engine
+        database.SESSIONMAKER = self._old_session
+
+        global _SETTINGS
+        _SETTINGS = self._old_settings
 
 
 def reset(env_file: Optional[Union[str, Tuple[str]]] = None) -> None:
     """Reset cacholote settings.
 
     Priority:
-    1. Evironment variables with prefix `CACHOLOTE_`
+    1. Environment variables with prefix `CACHOLOTE_`
     2. Dotenv file(s)
     3. Cacholote defaults
 
@@ -197,13 +174,16 @@ def reset(env_file: Optional[Union[str, Tuple[str]]] = None) -> None:
     env_file: str, tuple[str], default=None
         Dot env file(s).
     """
-    SETTINGS.set(Settings(_env_file=env_file))
+    global _SETTINGS
+    _SETTINGS = Settings(_env_file=env_file)
     set()
 
 
 def get() -> Settings:
     """Get cacholote settings."""
-    return SETTINGS.get()
+    if _SETTINGS is None:
+        raise ValueError(_CONFIG_NOT_SET_MSG)
+    return _SETTINGS.copy()
 
 
 reset()
