@@ -15,6 +15,7 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.import hashlib
 
+import datetime
 import hashlib
 import io
 import time
@@ -23,7 +24,6 @@ from types import TracebackType
 from typing import Any, Optional, Tuple, Type
 
 import fsspec
-import sqlalchemy.orm
 
 from . import config
 
@@ -31,14 +31,14 @@ from . import config
 def hexdigestify(text: str) -> str:
     """Convert text to its hash made of hexadecimal digits."""
     hash_req = hashlib.sha3_224(text.encode())
-    return hash_req.hexdigest()
+    return hash_req.hexdigest()[:32]
 
 
 def get_cache_files_fs_dirname() -> Tuple[fsspec.AbstractFileSystem, str]:
     """Return the ``fsspec`` filesystem and directory name where cache files are stored."""
     fs, _, (path,) = fsspec.get_fs_token_paths(
-        config.SETTINGS.get().cache_files_urlpath,
-        storage_options=config.SETTINGS.get().cache_files_storage_options,
+        config.get().cache_files_urlpath,
+        storage_options=config.get().cache_files_storage_options,
     )
     return (fs, path)
 
@@ -68,10 +68,16 @@ def copy_buffered_file(
 
 
 class _Locker:
-    def __init__(self, fs: fsspec.AbstractFileSystem, urlpath: str) -> None:
+    def __init__(
+        self,
+        fs: fsspec.AbstractFileSystem,
+        urlpath: str,
+        lock_validity_period: Optional[float] = None,
+    ) -> None:
         self.fs = fs
         self.urlpath = urlpath
         self.lockfile = urlpath + ".lock"
+        self.lock_validity_period = lock_validity_period
 
     @property
     def file_exists(self) -> bool:
@@ -84,9 +90,22 @@ class _Locker:
         if self.fs.exists(self.lockfile):
             self.fs.rm(self.lockfile)
 
+    @property
+    def is_locked(self) -> bool:
+        if not self.fs.exists(self.lockfile):
+            return False
+
+        delta = datetime.datetime.now() - self.fs.modified(self.lockfile)
+        if self.lock_validity_period is None or delta < datetime.timedelta(
+            seconds=self.lock_validity_period
+        ):
+            return True
+
+        return False
+
     def wait_until_released(self) -> None:
         warned = False
-        while self.fs.exists(self.lockfile):
+        while self.is_locked:
             if not warned:
                 warnings.warn(
                     f"can NOT proceed until file is released: {self.lockfile!r}.",
@@ -107,10 +126,3 @@ class _Locker:
         exc_tb: Optional[TracebackType],
     ) -> None:
         self.release()
-
-
-def _commit_or_rollback(session: sqlalchemy.orm.Session) -> None:
-    try:
-        session.commit()
-    finally:
-        session.rollback()
