@@ -1,13 +1,11 @@
-import contextvars
 import datetime
 import pathlib
-import threading
 import time
 from typing import Any
 
 import pytest
 
-from cacholote import cache, config
+from cacholote import cache, config, database
 
 
 def func(a: Any, *args: Any, b: Any = None, **kwargs: Any) -> Any:
@@ -32,8 +30,7 @@ def cached_error() -> None:
 
 
 def test_cacheable(tmpdir: pathlib.Path) -> None:
-
-    con = config.ENGINE.get().raw_connection()
+    con = config.get().engine.raw_connection()
     cur = con.cursor()
 
     cfunc = cache.cacheable(func)
@@ -44,10 +41,13 @@ def test_cacheable(tmpdir: pathlib.Path) -> None:
         after = datetime.datetime.utcnow()
         assert res == {"a": "test", "args": [], "b": None, "kwargs": {}}
 
-        cur.execute("SELECT key, expiration, result, counter FROM cache_entries", ())
+        cur.execute(
+            "SELECT id, key, expiration, result, counter FROM cache_entries", ()
+        )
         assert cur.fetchall() == [
             (
-                "a8260ac3cdc1404aa64a6fb71e85304922e86bcab2eeb6177df5c933",
+                1,
+                "a8260ac3cdc1404aa64a6fb71e853049",
                 "9999-12-31 23:59:59.999999",
                 '{"a": "test", "b": null, "args": [], "kwargs": {}}',
                 counter,
@@ -77,7 +77,6 @@ def test_encode_errors(tmpdir: pathlib.Path, raise_all_encoding_errors: bool) ->
         with pytest.warns(UserWarning, match="can NOT encode python call"):
             res = cfunc(inst)
         assert res == {"a": inst, "args": (), "b": None, "kwargs": {}}
-        assert cache.LAST_PRIMARY_KEYS.get() == {}
 
     if raise_all_encoding_errors:
         with pytest.raises(AttributeError):
@@ -86,21 +85,27 @@ def test_encode_errors(tmpdir: pathlib.Path, raise_all_encoding_errors: bool) ->
         with pytest.warns(UserWarning, match="can NOT encode output"):
             res = cfunc("test", b=1)
         assert res.__class__.__name__ == "LocalClass"
-        assert cache.LAST_PRIMARY_KEYS.get() == {}
 
     # cache-db must be empty
-    con = config.ENGINE.get().raw_connection()
+    con = config.get().engine.raw_connection()
     cur = con.cursor()
     cur.execute("SELECT * FROM cache_entries", ())
     assert cur.fetchall() == []
 
 
-def test_hexdigestify_python_call() -> None:
-    assert (
-        cache.hexdigestify_python_call(func, 1)
-        == cache.hexdigestify_python_call(func, a=1)
-        == "54f546036ae7dccdd0155893189154c029803b1f52a7bf5e6283296c"
-    )
+def test_same_args_kwargs() -> None:
+    ufunc = cache.cacheable(func)
+
+    con = config.get().engine.raw_connection()
+    cur = con.cursor()
+
+    ufunc(1)
+    cur.execute("SELECT id, key, counter FROM cache_entries", ())
+    assert cur.fetchall() == [(1, "54f546036ae7dccdd0155893189154c0", 1)]
+
+    ufunc(a=1)
+    cur.execute("SELECT id, key, counter FROM cache_entries", ())
+    assert cur.fetchall() == [(1, "54f546036ae7dccdd0155893189154c0", 2)]
 
 
 @pytest.mark.parametrize("use_cache", [True, False])
@@ -109,38 +114,35 @@ def test_use_cache(use_cache: bool) -> None:
 
     if use_cache:
         assert cached_now() == cached_now()
-        assert cache.LAST_PRIMARY_KEYS.get() == {
-            "key": "c3d9e414d0d32337c3672cb29b1b3cc9408001bf2d1b2a71c5e45fb6",
-            "expiration": datetime.datetime(9999, 12, 31, 23, 59, 59, 999999),
-        }
     else:
         assert cached_now() < cached_now()
-        assert cache.LAST_PRIMARY_KEYS.get() == {}
 
 
-def test_expiration() -> None:
-    first = cached_now()
-    assert cache.LAST_PRIMARY_KEYS.get() == {
-        "key": "c3d9e414d0d32337c3672cb29b1b3cc9408001bf2d1b2a71c5e45fb6",
-        "expiration": datetime.datetime(9999, 12, 31, 23, 59, 59, 999999),
-    }
+def test_expiration_and_return_cache_entry() -> None:
+    config.set(return_cache_entry=True)
+    first: database.CacheEntry = cached_now()  # type: ignore[assignment]
+    assert first.id == 1
+    assert first.key == "c3d9e414d0d32337c3672cb29b1b3cc9"
+    assert first.expiration == datetime.datetime(9999, 12, 31, 23, 59, 59, 999999)
 
-    with config.set(expiration="1908-03-09T00:00:00"):
-        assert cached_now() != first
-        assert cache.LAST_PRIMARY_KEYS.get() == {
-            "key": "c3d9e414d0d32337c3672cb29b1b3cc9408001bf2d1b2a71c5e45fb6",
-            "expiration": datetime.datetime(1908, 3, 9),
-        }
+    expiration = datetime.datetime.utcnow() + datetime.timedelta(seconds=0.1)
+    with config.set(expiration=expiration):
+        second: database.CacheEntry = cached_now()  # type: ignore[assignment]
+        assert second.result != first.result
+        assert second.id == 2
+        assert second.key == "c3d9e414d0d32337c3672cb29b1b3cc9"
+        assert second.expiration == expiration
 
-    assert first == cached_now()
-    assert cache.LAST_PRIMARY_KEYS.get() == {
-        "key": "c3d9e414d0d32337c3672cb29b1b3cc9408001bf2d1b2a71c5e45fb6",
-        "expiration": datetime.datetime(9999, 12, 31, 23, 59, 59, 999999),
-    }
+    time.sleep(0.1)
+    third: database.CacheEntry = cached_now()  # type: ignore[assignment]
+    assert third.result == first.result
+    assert third.id == 1
+    assert third.key == "c3d9e414d0d32337c3672cb29b1b3cc9"
+    assert third.expiration == datetime.datetime(9999, 12, 31, 23, 59, 59, 999999)
 
 
 def test_tag(tmpdir: pathlib.Path) -> None:
-    con = config.ENGINE.get().raw_connection()
+    con = config.get().engine.raw_connection()
     cur = con.cursor()
 
     cached_now()
@@ -165,21 +167,8 @@ def test_tag(tmpdir: pathlib.Path) -> None:
     assert cur.fetchall() == [("2", 4)]
 
 
-def test_contextvar() -> None:
-    cache.LAST_PRIMARY_KEYS.set({})
-
-    ctx = contextvars.copy_context()
-    ctx.run(cached_now)
-    assert ctx[cache.LAST_PRIMARY_KEYS] == {
-        "key": "c3d9e414d0d32337c3672cb29b1b3cc9408001bf2d1b2a71c5e45fb6",
-        "expiration": datetime.datetime(9999, 12, 31, 23, 59, 59, 999999),
-    }
-
-    assert cache.LAST_PRIMARY_KEYS.get() == {}
-
-
 def test_cached_error() -> None:
-    con = config.ENGINE.get().raw_connection()
+    con = config.get().engine.raw_connection()
     cur = con.cursor()
 
     with pytest.raises(ValueError, match="test error"):
@@ -187,34 +176,3 @@ def test_cached_error() -> None:
 
     cur.execute("SELECT * FROM cache_entries", ())
     assert cur.fetchall() == []
-
-
-def test_context_argument() -> None:
-    ctx = contextvars.copy_context()
-    assert cached_now() == cached_now(__context__=ctx)  # type: ignore[call-arg]
-
-
-@pytest.mark.parametrize("set_cache", ["cads"], indirect=True)
-def test_concurrent(set_cache: str) -> None:
-    @cache.cacheable
-    def cached_sleep(sleep: float) -> Any:
-        time.sleep(sleep)
-        return sleep
-
-    # Threading
-    ctx = contextvars.copy_context()
-    sleep = 0.2
-    t1 = threading.Timer(0, cached_sleep, args=(sleep,), kwargs={"__context__": ctx})
-    t2 = threading.Timer(
-        sleep / 2, cached_sleep, args=(sleep,), kwargs={"__context__": ctx}
-    )
-    t1.start()
-    t2.start()
-    t1.join()
-    t2.join()
-
-    # Check hits
-    con = config.ENGINE.get().raw_connection()
-    cur = con.cursor()
-    cur.execute("SELECT counter FROM cache_entries", ())
-    assert cur.fetchall() == [(2,)]
