@@ -1,4 +1,4 @@
-import contextvars
+import hashlib
 import importlib
 import io
 import pathlib
@@ -29,12 +29,12 @@ def test_dictify_io_object(tmpdir: pathlib.Path, io_delete_original: bool) -> No
     # Create file
     tmpfile = tmpdir / "test.txt"
     fsspec.filesystem("file").pipe_file(tmpfile, b"test")
-    tmp_checksum = fsspec.filesystem("file").checksum(tmpfile)
+    tmp_hash = f"{fsspec.filesystem('file').checksum(tmpfile):x}"
 
     # Check dict and cached file
     actual = extra_encoders.dictify_io_object(open(tmpfile, "rb"))
-    href = f"{readonly_dir}/{tmp_checksum}.txt"
-    local_path = f"{tmpdir}/cache_files/{tmp_checksum}.txt"
+    href = f"{readonly_dir}/{tmp_hash}.txt"
+    local_path = f"{tmpdir}/cache_files/{tmp_hash}.txt"
     checksum = fsspec.filesystem("file").checksum(local_path)
     expected = {
         "type": "python_call",
@@ -64,7 +64,8 @@ def test_dictify_bytes_io_object(
     tmpdir: pathlib.Path, obj: Union[io.BytesIO, io.StringIO]
 ) -> None:
     actual = extra_encoders.dictify_io_object(obj)["args"]
-    local_path = f"{tmpdir}/cache_files/{hash(obj)}"
+    obj_hash = hashlib.md5(f"{hash(obj)}".encode()).hexdigest()
+    local_path = f"{tmpdir}/cache_files/{obj_hash}"
     checksum = fsspec.filesystem("file").checksum(local_path)
     expected: Tuple[Dict[str, Any], ...] = (
         {
@@ -86,15 +87,14 @@ def test_copy_from_http_to_cache(
     httpserver: pytest_httpserver.HTTPServer,
     set_cache: str,
 ) -> None:
-
     # cache-db to check
-    con = config.ENGINE.get().raw_connection()
+    con = config.get().engine.raw_connection()
     cur = con.cursor()
 
     # http server
     httpserver.expect_request("/test").respond_with_data(b"test")
     url = httpserver.url_for("/test")
-    cached_basename = str(fsspec.filesystem("http").checksum(url))
+    cached_basename = f"{fsspec.filesystem('http').checksum(url):x}"
 
     # cache http file
     fs, dirname = utils.get_cache_files_fs_dirname()
@@ -117,11 +117,10 @@ def test_copy_from_http_to_cache(
 def test_io_corrupted_files(
     tmpdir: pathlib.Path, httpserver: pytest_httpserver.HTTPServer
 ) -> None:
-
     # http server
     httpserver.expect_request("/test").respond_with_data(b"test")
     url = httpserver.url_for("/test")
-    cached_basename = str(fsspec.filesystem("http").checksum(url))
+    cached_basename = f"{fsspec.filesystem('http').checksum(url):x}"
 
     # cache file
     fs, dirname = utils.get_cache_files_fs_dirname()
@@ -153,16 +152,15 @@ def test_io_locker_warning(tmpdir: pathlib.Path) -> None:
 
     # Acquire lock
     fs, dirname = utils.get_cache_files_fs_dirname()
-    checksum = fsspec.filesystem("file").checksum(tmpfile)
-    lock = f"{dirname}/{checksum}.txt.lock"
+    file_hash = f"{fsspec.filesystem('file').checksum(tmpfile):x}"
+    lock = f"{dirname}/{file_hash}.txt.lock"
     fs.touch(lock)
 
     def release_lock(fs: fsspec.AbstractFileSystem, lock: str) -> None:
         fs.rm(lock)
 
     # Threading
-    ctx = contextvars.copy_context()
-    t1 = threading.Timer(0, cfunc, args=(tmpfile,), kwargs={"__context__": ctx})
+    t1 = threading.Timer(0, cfunc, args=(tmpfile,))
     t2 = threading.Timer(0.1, release_lock, args=(fs, lock))
     with pytest.warns(
         UserWarning, match=f"can NOT proceed until file is released: {lock!r}."
