@@ -20,24 +20,24 @@ def test_clean_cache_files(
     set_cache: str,
     method: Literal["LRU", "LFU"],
 ) -> None:
-
-    con = config.ENGINE.get().raw_connection()
+    con = config.get().engine.raw_connection()
     cur = con.cursor()
     fs, dirname = utils.get_cache_files_fs_dirname()
 
     # Create files
-    checksums = {}
+    paths = {}
     for algorithm in ("LRU", "LFU"):
         filename = tmpdir / f"{algorithm}.txt"
         fsspec.filesystem("file").pipe_file(filename, b"1")
-        cachedname = f"{dirname}/{fsspec.filesystem('file').checksum(filename)}.txt"
-        checksums[algorithm] = cachedname
+        file_hash = f"{fsspec.filesystem('file').checksum(filename):x}"
+        cachedname = f"{dirname}/{file_hash}.txt"
+        paths[algorithm] = cachedname
 
     # Copy to cache
     open_url(tmpdir / "LRU.txt")
     open_url(tmpdir / "LRU.txt")
     open_url(tmpdir / "LFU.txt")
-    assert set(fs.ls(dirname)) == set(checksums.values())
+    assert set(fs.ls(dirname)) == set(paths.values())
 
     # Do not clean
     clean.clean_cache_files(2, method=method)
@@ -50,14 +50,11 @@ def test_clean_cache_files(
     cur.execute("SELECT * FROM cache_entries", ())
     nrows = len(cur.fetchall())
     assert nrows == fs.du(dirname) == 1
-    assert not fs.exists(f"{dirname}/{checksums[method]}.txt")
+    assert not fs.exists(f"{dirname}/{paths[method]}.txt")
 
 
 @pytest.mark.parametrize("delete_unknown_files", [True, False])
-@pytest.mark.parametrize("add_lock", [True, False])
-def test_delete_unknown_files(
-    tmpdir: pathlib.Path, delete_unknown_files: bool, add_lock: bool
-) -> None:
+def test_delete_unknown_files(tmpdir: pathlib.Path, delete_unknown_files: bool) -> None:
     fs, dirname = utils.get_cache_files_fs_dirname()
 
     # Create file
@@ -69,21 +66,45 @@ def test_delete_unknown_files(
 
     # Add unknown
     fs.put(str(tmpfile), f"{dirname}/unknown.txt")
-    if add_lock:
-        fs.touch(f"{dirname}/unknown.txt.lock")
 
     # Clean one file
     clean.clean_cache_files(1, delete_unknown_files=delete_unknown_files)
-    if delete_unknown_files and not add_lock:
-        assert fs.ls(dirname) == [f"{dirname}/{fs.checksum(tmpfile)}.txt"]
+    if delete_unknown_files:
+        file_hash = f"{fs.checksum(tmpfile):x}"
+        assert fs.ls(dirname) == [f"{dirname}/{file_hash}.txt"]
     else:
-        if add_lock:
-            assert set(fs.ls(dirname)) == {
-                f"{dirname}/unknown.txt",
-                f"{dirname}/unknown.txt.lock",
-            }
-        else:
-            assert fs.ls(dirname) == [f"{dirname}/unknown.txt"]
+        assert fs.ls(dirname) == [f"{dirname}/unknown.txt"]
+
+
+@pytest.mark.parametrize("lock_validity_period", [None, 0])
+def test_clean_locked_files(
+    tmpdir: pathlib.Path, lock_validity_period: Optional[float]
+) -> None:
+    fs, dirname = utils.get_cache_files_fs_dirname()
+
+    # Create file
+    tmpfile = tmpdir / "test.txt"
+    fsspec.filesystem("file").pipe_file(tmpfile, b"1")
+
+    # Copy to cache
+    open_url(tmpfile)
+
+    # Add unknown and lock
+    fs.put(str(tmpfile), f"{dirname}/unknown.txt")
+    fs.touch(f"{dirname}/unknown.txt.lock")
+
+    # Clean one file
+    clean.clean_cache_files(
+        1, delete_unknown_files=True, lock_validity_period=lock_validity_period
+    )
+    if lock_validity_period == 0:
+        file_hash = f"{fs.checksum(tmpfile):x}"
+        assert fs.ls(dirname) == [f"{dirname}/{file_hash}.txt"]
+    else:
+        assert set(fs.ls(dirname)) == {
+            f"{dirname}/unknown.txt",
+            f"{dirname}/unknown.txt.lock",
+        }
 
 
 @pytest.mark.parametrize(
@@ -112,9 +133,8 @@ def test_clean_tagged_files(
             open_url(tmpfile)
 
     clean.clean_cache_files(1, tags_to_clean=tags_to_clean, tags_to_keep=tags_to_keep)
-    assert fs.ls(dirname) == [
-        f"{dirname}/{fs.checksum(tmpdir / f'test_{expected}.txt')}.txt"
-    ]
+    file_hash = f"{fs.checksum(tmpdir / f'test_{expected}.txt'):x}"
+    assert fs.ls(dirname) == [f"{dirname}/{file_hash}.txt"]
 
 
 def test_clean_tagged_files_wrong_args() -> None:
